class Error(Exception):
    """Base exception for LuluTest"""
    pass


class PageNotLoadedError(Error):
    """
    When user tries to do driver actions that require a page
    to have been "gone to." We don't want to fail this silently
    to avoid false positives.
    """
    pass


class NoElementWithNameInPage(Error):
    """
    Raise this error when user tries to get element by name
    from a page object, but an element with that name does
    not exist.
    """
    pass


class TooManyElementsWithNameInPage(Error):
    """
    Raise this error when user tries to get element by name
    from a page object, but there are multiple elements with
    that name.
    """
    pass



from LuluTest.action.page_actions import PageActions
from LuluTest.lulu_exceptions import PageNotLoadedError

'''
This class contains meta-programming to decorate the
PageActions class methods with the necessary pre and
post processing functionality. Proceed with caution.
'''


class PageActionsDecorator(object):
    FUNCTIONS_NEEDING_PAGE_TO_BE_LOADED = [
        'refresh',
        'get_page_source',
        'get_url',
        'execute_script'
    ]

    FUNCTIONS_THAT_LOAD_THE_PAGE = [
        'go',
        'go_to',
        'close'
    ]

    LOAD_WEB_DRIVER_MESSAGE = "Web Driver must go to page " \
                              "before performing this action"

    def __init__(self, model: PageActions):
        self.model = model

    def __getattr__(self, func):
        def method(*args):
            if func in self.FUNCTIONS_NEEDING_PAGE_TO_BE_LOADED:
                return self.check_page_load(func, *args)
            if func in self.FUNCTIONS_THAT_LOAD_THE_PAGE:
                return self.set_page_loaded(func, *args)
        return method

    def check_page_load(self, func, *args):
        if not self.model.page_loaded:
            raise PageNotLoadedError(self.LOAD_WEB_DRIVER_MESSAGE)
        else:
            return getattr(self.model, func)(*args)

    def set_page_loaded(self, func, *args):
        getattr(self.model, func)(*args)
        if func == 'close':
            self.model.page_loaded = False
        else:
            self.model.page_loaded = True



import unittest

from LuluTest.page import Page
from LuluTest.element import PageElement, AlertElement
from LuluTest.action import Action


class TestOtherPages(unittest.TestCase):

    def test_saucedemo_refresh_page(self):
        page = Page('https://www.saucedemo.com/inventory.html')
        actions = Action()
        actions.go(page)

        page.elements = [
            PageElement(("xpath", "(//button[@class='btn_primary btn_inventory'])[1]"), "button 1"),
            PageElement(("xpath", "(//div[@class='inventory_item_name'])[1]"), "inventory item"),
            PageElement(("class", "shopping_cart_badge"), "shopping cart badge"),
            PageElement(("class", "inventory_item_name"), "item name")
        ]

        actions.click(page.get_element("button 1"))
        item_name = actions.get_attribute(page.get_element("inventory item"), "innerHTML")
        items_in_cart = actions.get_attribute(page.get_element("shopping cart badge"), "innerHTML")

        self.assertEqual(items_in_cart, "1")
        actions.refresh()
        items_in_cart = actions.get_attribute(page.get_element("shopping cart badge"), "innerHTML")
        self.assertEqual(items_in_cart, "1")

        actions.go_to('https://www.saucedemo.com/cart.html')
        inventory_item_name = actions.get_attribute(page.get_element("item name"), "innerHTML")
        self.assertEqual(item_name, inventory_item_name)
        actions.close()

    def test_javascript_alert(self):
        page = Page('https://the-internet.herokuapp.com/javascript_alerts')
        actions = Action()
        actions.go(page)

        to_be_clicked = PageElement(('xpath', "(//button)[1]"))
        actions.click(to_be_clicked)
        alert = AlertElement()
        js_text = actions.get_text(alert)
        self.assertEqual(js_text, "I am a JS Alert")
        actions.accept(alert)
        actions.close()

    def test_javascript_confirm(self):
        page = Page('https://the-internet.herokuapp.com/javascript_alerts')
        actions = Action()
        actions.go(page)

        to_be_clicked = PageElement(('xpath', "(//button)[2]"))
        actions.click(to_be_clicked)
        alert = AlertElement()
        js_text = actions.get_text(alert)
        self.assertEqual(js_text, "I am a JS Confirm")
        actions.dismiss(alert)
        actions.close()

    def test_javascript_prompt(self):
        page = Page('https://the-internet.herokuapp.com/javascript_alerts')
        actions = Action()
        actions.go(page)
        page.elements = [
            PageElement(("xpath", "(//button)[3]"), "prompt button"),
            PageElement(("id", "result"), "result"),
            AlertElement('alert')
        ]

        actions.click(page.get_element("prompt button"))
        alert_text = actions.get_text(page.get_element('alert'))
        self.assertEqual(alert_text, "I am a JS prompt")
        actions.input_text(page.get_element('alert'), 'This is a test')
        actions.accept(page.get_element('alert'))

        result = actions.get_attribute(page.get_element('result'), 'innerHTML')
        self.assertEqual(result, "You entered: This is a test")
        actions.close()

    def test_file_upload(self):
        import os  # Need this to work locally and in remote CI
        base_path = os.getcwd()
        if 'tests' in base_path:
            pass
        else:
            base_path += '/tests'

        page = Page('http://the-internet.herokuapp.com/upload')
        actions = Action()
        actions.go(page)
        page.elements = [
            PageElement(('id', 'file-upload'), 'Upload Element'),
            PageElement(('id', 'file-submit'), 'Submit Button')
        ]
        file_path = base_path + '/fixtures/files/upload_text_file.txt'
        actions.upload_file(page.get_element('Upload Element'), file_path)
        actions.click(page.get_element('Submit Button'))

        success_text = 'File Uploaded!'
        self.assertIn(success_text, actions.get_page_source())

    def test_dropdown(self):
        page = Page('http://the-internet.herokuapp.com/dropdown')
        actions = Action()
        actions.go(page)
        dropdown = PageElement(('id', 'dropdown'), 'dropdown')
        option_2 = PageElement(('xpath', '/html/body/div[2]/div/div/select/option[3]'), 'option 2')
        actions.select_drop_down(dropdown, 'Option 2')
        is_selected = actions.get_attribute(option_2, 'selected')
        self.assertTrue(is_selected)

    def test_script_execution(self):
        page = Page('https://demoqa.com/accordian/')
        actions = Action()
        actions.go(page)
        click_header_script = 'document.getElementById("section2Heading").click()'
        div_class_name = 'return document.getElementById("section2Content").parentElement.className'
        self.assertEqual(actions.execute_script(div_class_name), 'collapse')
        actions.execute_script(click_header_script)
        self.assertEqual(actions.execute_script(div_class_name), 'collapsing')



class Page:
    def __init__(self, url, elements=None):
        if elements is None:
            elements = []
        self.url = url
        self.elements = elements

    def get_element(self, getter):
        element = None
        if isinstance(getter, int):
            element = self.elements[getter]
        elif isinstance(getter, str):
            elements_found_array = [elem for elem in self.elements if elem.name == getter]
            element = self.resolve_found_elements(elements_found_array)
        return element

    @staticmethod
    def resolve_found_elements(elements):
        size = len(elements)
        if size == 1:
            return elements[0]
        elif size == 0:
            raise NoElementWithNameInPage
        elif size > 1:
            raise TooManyElementsWithNameInPage



from LuluTest.element import BaseElement
from .step import Step


class Steps:
    def __init__(self, action_object, step_builders):
        self.action_object = action_object
        self.each = []
        for builder in step_builders:
            self.each.append(self.__resolve_step(builder))

    def __resolve_step(self, step):
        if isinstance(step, BaseElement):
            return step

        resolved_step = None
        if len(step) == 2:
            resolved_step = Step(
                action_object=self.action_object,
                operation=step[0],
                subject=step[1]
            )
        elif len(step) == 3:
            resolved_step = Step(
                action_object=self.action_object,
                operation=step[0],
                subject=step[1],
                data=step[2]
            )
        return resolved_step



class PageActions:
    def __init__(self, driver):
        self.driver = driver
        self.page_loaded = False

    def go(self, page):
        self.driver.get(page.url)

    def go_to(self, url):
        self.driver.get(url)

    def close(self):
        self.driver.close()

    def refresh(self):
        self.driver.refresh()

    def get_page_source(self):
        return self.driver.page_source

    def get_url(self):
        return self.driver.current_url

    def execute_script(self, script):
        return self.driver.execute_script(script)



from .action import Action
from .page_actions import PageActions
from .element_actions import ElementActions

page_action_map = {
    "go": PageActions.go,
    "go to": PageActions.go_to,
    "close": PageActions.close,
    "refresh": PageActions.refresh,
    "get source": PageActions.get_page_source,
    "get url": PageActions.get_url
}

element_action_map = {
    "click": ElementActions.click,
    "type": ElementActions.input_text,
    "clear": ElementActions.clear,
    "clear text": ElementActions.clear_text,
    "select": ElementActions.select_drop_down,
    "get attribute": ElementActions.get_element_attribute,
    "check text": ElementActions.check_element_text,
    "accept": ElementActions.accept,
    "dismiss": ElementActions.dismiss
}



from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.firefox.options import Options


def new(browser_type, options):
    browser_type = browser_type.lower()
    browser_function = eval('__{}_driver'.format(browser_type))
    return browser_function(options)


def __chrome_driver(options):
    chrome_options = webdriver.chrome.options.Options()
    if 'not headless' not in options:
        chrome_options.add_argument('--headless')
    return webdriver.Chrome(options=chrome_options)


def __firefox_driver(options):
    firefox_options = Options()
    if 'not headless' not in options:
        firefox_options.headless = True
    return webdriver.Firefox(options=firefox_options)



from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select

from LuluTest.page_element_interface.IPageElement import load_element, check_element_text


class ElementActions:
    def __init__(self, driver):
        self.driver = driver

    def click(self, element):
        driver_element = load_element(self.driver, element)
        driver_element.click()

    def input_text(self, element, text):
        driver_element = load_element(self.driver, element)
        driver_element.send_keys(text)

    def clear(self, element):
        driver_element = load_element(self.driver, element)
        driver_element.clear()

    def clear_text(self, element):
        driver_element = load_element(self.driver, element)
        driver_element.send_keys(Keys.CONTROL + 'a')
        driver_element.send_keys(Keys.DELETE)

    def select_drop_down(self, element, visible_text):
        driver_element = load_element(self.driver, element)
        selectable = Select(driver_element)
        selectable.select_by_visible_text(visible_text)

    def upload_file(self, element, path):
        self.input_text(element, path)

    def get_element_attribute(self, element, attribute):
        driver_element = load_element(self.driver, element)
        return driver_element.get_attribute(attribute)

    def get_text(self, element):
        driver_element = load_element(self.driver, element)
        return driver_element.text

    def check_element_text(self, element, text):
        driver_element = check_element_text(self.driver, element, text)
        return driver_element

    def get_attribute(self, element, attribute):
        driver_element = load_element(self.driver, element)
        return driver_element.get_attribute(attribute)

    def accept(self, element):
        driver_element = load_element(self.driver, element)
        driver_element.accept()

    def dismiss(self, element):
        driver_element = load_element(self.driver, element)
        driver_element.dismiss()



import unittest

from LuluTest.action import Action
from step import Do, Steps
from LuluTest.page import page_factory


class PrebuiltPageTest(unittest.TestCase):
    import os  # Need this to work locally and in remote CI
    base_path = os.getcwd()
    if 'tests' in base_path:
        pass
    else:
        base_path += '/tests'

    prebuilt_pages_directory = base_path + '/fixtures/pages/'
    page_configs = [
        prebuilt_pages_directory + 'news_outlet.json',
        prebuilt_pages_directory + 'inventory.yml'
    ]
    pages = page_factory.generate_pages(page_configs)

    def test_basic_usage(self):
        page = self.pages['news_outlet']
        actions = Action()
        actions.go(page)
        actions.input_text(page.get_element("input_box"), "Hello")
        actions.click(page.get_element("button"))
        english_div = page.get_element("english_div")
        english_text = actions.check_element_text(english_div, "Hello")
        self.assertTrue(english_text)
        actions.close()

    def test_complicated_page(self):
        page = self.pages['inventory']
        actions = Action()
        actions.go(page)

        steps = Steps(actions, [
            ('click', page.get_element('inv_btn1')),
            ('click', page.get_element('inv_btn2')),
            ('click', page.get_element('inv_btn3'))
        ])

        Do(steps)
        items_in_cart = actions.get_attribute(page.get_element("Cart Badge"), "innerHTML")
        self.assertEqual(items_in_cart, "3")

        actions.click(page.get_element("Cart Link"))
        actions.click(page.get_element("Cart Button"))
        items_in_cart = actions.get_attribute(page.get_element("Cart Badge"), "innerHTML")
        self.assertEqual(items_in_cart, "2")
        actions.click(page.get_element("Checkout Button"))

        steps = Steps(actions, [
            ('type', page.get_element('First Name'), 'Jane'),
            ('type', page.get_element('Last Name'), 'Doe'),
            ('type', page.get_element('Zip'), '12345'),
            ('click', page.get_element('Cart Button'))
        ])
        Do(steps)

        actions.click(page.get_element("Cart Button"))
        checkout_complete_page = actions.get_attribute(page.get_element("Completion Header"), "innerHTML")
        self.assertEqual(checkout_complete_page, "THANK YOU FOR YOUR ORDER")

        actions.close()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='LuluTest',
    version='0.1.0',
    author='Erik Whiting',
    author_email='erik@erikwhiting.com',
    description='A web browser automation framework',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/erik-whiting/LuluTest',
    packages=setuptools.find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 3',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    python_requires='>=3.5'
)
